/************************************************************************
 *
 * $Id: PeerID.java,v 1.7 2002/01/03 01:04:37 bondolo Exp $
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *********************************************************************************/

package net.jxta.impl.id.UUID;

import java.net.URL;

/**
 * This class implements a PeerID. Each peer is assigned a unique peer id.UUID
 * id are used to implement peer id.
 *
 * @see         net.jxta.impl.id.UUID.UUID
 * @see         net.jxta.impl.id.UUID.UUIDFactory
 *
 * @version     $Revision: 1.7 $
 * @since       JXTA  1.0
 */
public final class PeerID extends net.jxta.peer.PeerID {
    
    final static int groupIdOffset = 0;
    final static int idOffset = PeerID.groupIdOffset + IDFormat.uuidSize;
    final static int padOffset = PeerID.idOffset + IDFormat.uuidSize;
    
    final static int padSize = IDFormat.flagsOffset - PeerID.padOffset;
    
    protected IDBytes id;
    
    /**
     *  Constructor. Used only internally.
     *
     * @since       JXTA  1.0
     **/
    private PeerID() {
        super();
        id = new IDBytes();
        id.bytes[IDFormat.flagsOffset + IDFormat.flagsIdTypeOffset] = IDFormat.flagPeerID;
    };
    
    /**
     * Constructor.
     * Intializes contents from provided ID.
     *
     * @since       JXTA  1.0
     *
     * @param id    the ID data
     **/
    PeerID( IDBytes id ) {
        super();
        this.id = id;
    };
    
    /**
     * Constructor.
     * Creates a PeerID. A PeerGroupID is provided
     *
     * @since       JXTA  1.0
     *
     * @param groupID    the group to which this will belong.
     **/
    public PeerID(PeerGroupID groupID) {
        this( );
        if(PeerGroupID == null) throw new RuntimeException("Dumb Ass");
        UUID    groupUUID = new UUID(
        groupID.id.bytesIntoLong( PeerGroupID.groupIdOffset ),
        groupID.id.bytesIntoLong( PeerGroupID.groupIdOffset + 8 ) );
        UUID    idUUID = UUIDFactory.newUUID();
        
        id.longIntoBytes(
        PeerID.groupIdOffset, groupUUID.getMostSignificantBits() );
        id.longIntoBytes(
        PeerID.groupIdOffset + 8, groupUUID.getLeastSignificantBits() );
        
        id.longIntoBytes(
        PeerID.idOffset, idUUID.getMostSignificantBits() );
        id.longIntoBytes(
        PeerID.idOffset + 8, idUUID.getLeastSignificantBits() );
    };
    
    /**
     * Constructor.
     * Creates a PeerID. A PeerGroupID is provided
     *
     * @param groupID    the group to which this will belong.
     * @param peerID     id of this peer
     *
     * @since       JXTA  1.0
     **/
    PeerID( UUID groupUUID, UUID peerUUID ) {
        this( );
        
        id.longIntoBytes(
        PeerID.groupIdOffset, groupUUID.getMostSignificantBits() );
        id.longIntoBytes(
        PeerID.groupIdOffset + 8, groupUUID.getLeastSignificantBits() );
        
        id.longIntoBytes(
        PeerID.idOffset, peerUUID.getMostSignificantBits() );
        id.longIntoBytes(
        PeerID.idOffset + 8, peerUUID.getLeastSignificantBits() );
    };
    
    /**
     * Clones this ID
     *
     * @since JXTA 1.0
     *
     * @return Object a clone object of this ID.
     **/
    public Object clone() {
        return this; // ID instances are immutable so we can return ourself
    }
    
    /**
     * Returns true if the two IDs are same
     *
     * @since JXTA 1.0
     *
     * @param target the object to be compared agaisnt.
     * @return boolean returns true if the objects are equal otherwise false.
     **/
    public boolean equals( Object target ) {
        if (this == target) {
            return true;
        }
        
        // FIXME 20011220 bondolo@jxta.org very many places try to compare
        // strings against ids. For now we allow this to work.
        if (target instanceof String )
            return target.equals( this.toString() );
        
        if (target instanceof PeerID )
            return id.equals( ((PeerID)target).id );
        else 
            return false;
    };
    
    /**
     * Public member calculates a hash code for this ID. Used by Hashmaps.
     *
     *  @since JXTA 1.0
     *
     * @return int Containing the hashcode of this ID.
     **/
    public int hashCode() {
        return id.hashCode();
    };
    
    /**
     *
     *  Returns a string identifier which indicates which ID format is
     *  used by this ID instance.
     *
     *  @since JXTA 1.0
     *
     *  @return	returns a string identifier which indicates which ID format is
     *  used by this ID instance.
     **/
    public String getIDFormat() {
        return IDFormat.INSTANTIATOR.getSupportedIDFormat();
    }
    
    /**
     *
     *  Return an object containing the unique value of the ID. This object must
     *  provide implementations of toString() and hashCode() that are canonical
     *  and conisistent from run-to-run given the same input values. Beyond
     *  this nothing should be assumed about the nature of this object. For some
     *  implementations the object returned may be the same as provided.
     *
     *  @since JXTA 1.0
     *
     *  @return	Object which can provide canonical representations of the ID.
     **/
    public Object getUniqueValue() {
        return getIDFormat() + "-" + (String) id.getUniqueValue();
    }

    /**
     * Public member which returns a URI (URL in Java nomenclature) of the ID.
     *
     *  @since JXTA 1.0
     *
     * @return	URL Object containing the URI
     **/
    public URL getURL( ) {
        return IDFormat.getURL( (String) getUniqueValue() );
    }
    
    /**
     * Returns the PeerGroupID of the PeerGroup associated with this Peer
     *
     * @since       JXTA  1.0
     *
     * @return PeerGroupID return peer group Id
     **/
    public net.jxta.id.ID getPeerGroupID() {
        UUID groupUUID = new UUID(
        id.bytesIntoLong( PeerID.groupIdOffset ),
        id.bytesIntoLong( PeerID.groupIdOffset + 8 ) );
        
        PeerGroupID groupID = new PeerGroupID( groupUUID );
        
        // convert to the generic world PGID as necessary
        return IDFormat.translateToWellKnown( groupID );
    };
    
    
}
